build status: [![Build Status](https://secure.travis-ci.org/binocarlos/telegraft.png)](http://travis-ci.org/binocarlos/telegraft)

##Caution!
This is still coming together so don't use it yet!

[![Telegraft Server Management](https://dl.dropbox.com/s/rs05ps3ipx5y3yt/telegraft.jpg?dl=1)](http://binocarlos.github.com/telegraft/)

#telegraft
Telegraft is a node.js orchestration framework - it is powerful, lightweight and refreshingly flexible.

It can:

* orchestrate the remote configuration of your server infrastructure

* manage the sequential execution of database tasks

* combine complicated sequences of pre-defined logic and apply them to variable targets in a phased manner

## Motivation
I was first loving [Puppet](http://puppetlabs.com/) and then [Chef](http://www.opscode.com/chef/) (especially it's wonderfully
metaphorical sense of wordplay and yes, I'm guilty of that here for shore)

Then I stumbled across [Ansible](http://ansible.github.com/) and really liked the way it just used an SSH connection to do things,
this struck me as sensible in the land of KISS.

However - I was dismayed at the restrictions imposed by having to **DO** things using plain ol' YAML config files and not being 
able to hook my own juicy node.js functions into it without going on a silly detour.

I was playing with wonderfully elegent node.js tools like [Connect](http://www.senchalabs.org/connect/) and [Express](http://expressjs.com/)
and thinking 'why can my orchestration library not be that simple'.

The main motive behind telegraft is to allow the powerful abilities of Puppet, Chef and Ansible but inside a framework that is
wonderfully expressive and minimal as Connect and Express (oh, and in node.js please : )

The power of functional programming and middleware is something I've only just grasped - 
hopefully this project is a winner and not a dogs dinner!

## Installation

      npm install telegraft

## Usage
Installing Apache on 3 servers:

	var telegraft = require('telegraft');

	var tx = new telegraft({

		modulators:{
			'ssh':telegraft.modulator.ssh
		}

	}).record(function(broadcast){

		broadcast.announce('Installing Apache');

		broadcast.record(telegraft.recordings.package({
			name:"httpd",
			action:"install"
		}));

	}).transmit([
		'1.2.3.4',
		'1.2.3.5',
		'1.2.3.6'
	]);

## Terminology
	
* Transmitter: 		the main stack that is setup before carrying out some tasks

* Recording:			an orchestration function that sequencies and prepares tasks

* Broadcast:			the execution of some recordings at some targets

* Frequency:			defines the target(s) for a broadcast

* Transmission:		a sequence of tasks for a single target

* Packet:					a single task for a single target

* Modulator: 			a worker function that accepts a packet and actions it somehow

* Middleware:			a chained function that operates on a packet before and after modulation

### Transmitter
The transmitter is the boss of the telegraft stack - it is what you interact with at the topmost level:

	var telegraft = require('telegraft');

	// make a new transmitter passing in a configuration
	var tx = new telegraft({

		// the name is useful for logging
		'name':'My First Tx',

		// we setup our modulators - things that know how to get data from A to B (like SSH & SCP)
		modulators:{

			// we enable the standard SSH modulator to connect to remote servers and run commands
			'ssh':telegraft.modulator.ssh

		}

	});

Once we have a transmitter (Tx), we need something to broadcast.

### Recording
A Recording is a function that accepts a **broadcast** object upon which it will call:

* announce: group the following sequence of transmissions by name

* transmit: send data using a particular modulation (ssh,scp,custom...)

* record: pass the broadcast over to another recording function

* wait: pause the whole broadcast until all preceeding recordings have transmitted to all frequencies

You can save recordings into external libraries for use amoungst projects and telegraft comes
with a whole bunch of useful recordings (like restart service, delete file, install MySQL etc).

#### Library Recordings
here is an example of a very simple (closure) recording that lists a certain directory:

	var listDirectory = function(dir) {

		// make the closure using 'dir' as the path
		return function(broadcast) {

			// the transmit function is the meat and gravy
			// it targets a named modulator in the transmitter
			broadcast.transmit('ssh', 'ls -la ' + dir);

		}

	}

we can now use this pre-recording to actually do stuff:

	tx.record(function(broadcast){

		// list some example directorys on each remote server
		broadcast.record(listDirectory('/etc'));
		broadcast.record(listDirectory('/usr/local/bin'));

	});

The above recording is a great example of a re-usable chunk of code (being in a closure) and can be saved
out into a module and used again.  How you do this is up to you, telegraft does not want to get involved
with that, it wants you to have the power over how you organize your recordings.

#### Orchestration Recordings
Not all recordings need to be library orientated - orchestration tasks like controlling
your infrastructure are better done ad-hoc:

	var rebuildApplicationRecording = function(broadcast){

		// announce a new group of transmissions
		broadcast.transmit('announce', 'Rebuilding Application Server');

		// first update the server from git
		broadcast.transmit('ssh', 'cd /usr/local/some/really/long/path; git update from my app repo...');

		// then rebuild the application
		broadcast.transmit('ssh', 'make rebuild');

		// then we wait for ALL servers to have finished the above
		broadcast.wait();

		// pass the broadcast on to another recording function
		broadcast.record(telegraft.recording.service({
			name:'myapp',
			action:'restart'
		}));

	}

#### Best Practice
The general pattern is for application infrastructure recordings (like deploy another database server) to live in with the
application - things get too specific to have these things as libraries.

You then have libraries that represent more common tasks (like add a virtual user to Apache).

A library recording is one that operates without any knowledge of context (move this file, add a user).

An orchestration recording is all about context and is the main thing you will be running from the top level.

### Frequency
Once you have some recordings, you are ready to broadcast them via the transmitter.

We need to give the broadcast some frequencies otherwise it won't end up anywhere.

A frequency can by ANY value - it is simply passed along to the modulator for reference.

In our server example, the frequencies would be be IP addresses:

	var servers = {
		'database':	['1.1.1.4', '1.1.1.8'],
		'web':			['4.4.2.2', '4.4.2.3']
	};

Here we have just represented our infrastructure - it's just some arrays of IP addresses and can be saved, 
loaded and manipulated any which way you want.

We can then broadcast to our servers:

	// standard
	var tx = new telegraft({
		modulators:{
			'ssh':telegraft.modulator.ssh
		}
	})
	// restart Apache
	.record(telegraft.recording.service({
		name:'httpd',
		action:'restart'
	}))
	// send to web servers
	.transmit(servers.web);

Above we targeted the web servers with the broadcast by giving the frequencies to **.transmit**.

You can change the frequencies at any time during a recording - this is useful for ad-hoc orchestration commands:

	var tx = new telegraft({
		modulators:{
			'ssh':telegraft.modulator.ssh
		}
	})
	.record(function(broadcast){

		// target the database servers
		broadcast.frequency(servers.database);

		// tell MySQL to restart
		broadcast.record(telegraft.recording.service({
			name:'mysql',
			action:'restart'
		}));

		// wait for both databases to finish restarting
		broadcast.wait();

		// target the web servers
		broadcast.frequency(servers.web);

		// tell Apache to restart
		broadcast.record(telegraft.recording.service({
			name:'httpd',
			action:'restart'
		}));

	})
	.transmit();

### Transmissions
A broadcast is a recording sent out to multiple frequencies.

A transmission is a recording sent out to a single frequency.

Where recordings are the main building blocks for preparing the broadcast, transmissions are the main building blocks
for co-ordinating actually sending it.

By splitting a broadcast into one transmission per frequency, we can control each transmission seperately (like halting
one if it went wrong but carrying on with the rest).

### Packets
Each transmission the transmitter makes (the number of which decided by how many frequencies it is broadcast on) - will
result in a sequence of packets being created.

A packet represents one transmission being sent to one frequency - in our server example a packet could be:

	{
		modulator: 	"ssh",
		data: 			"ls -la /etc",
		frequency:	"1.2.3.4"
	}

This would end up in the following SSH command:

	ssh user@1.2.3.4 ls -la /etc

In this example, the frequency the packet was sent to is: 1.2.3.4 and the modulator was SSH.

Each packet consists of:

* data: 				the data the modulator should send

* frequency: 		the target for the data (IP address of server but can be any value)

* transmission: the current sequence this packet is a part of

* result: 			a value representing the reply from the modulator

* error: 				a value representing the error from the modulator

* store: 				a common object present in all packets for that particular frequency

* modulator: 		the routing key that lead the packet to here

* id: 					a unique id for the packet across the whole broadcast

Each packet will be part of a transmission.

### Middleware
The transmitter has two main middleware stacks: **before** and **after**, a packet is run through
each either side of being modulated.

The before middleware stack is useful for filtering and adding data to the store to be used later on:

	// tells us if we can run the command
	var shouldIgnoreSSHCommand = function(hostname, command){
		..
	};

	// middleware to ignore certain SSH commands by hostname
	transmitter.before(function(packet, next){
		if(shouldIgnoreSSHCommand(packet.frequency, packet.data)) {
			packet.destroy();
		}
		
		next();
	});
	
The after middleware is useful for logging because the packet now contains the error - we can also
can **halt** on the transmission if thats how we roll:

	// middleware to halt the transmission on any error
	transmitter.after(function(packet, next){
		if(packet.error){
			packet.transmission.halt();
		}
		
		next();
	});
		
You can also use the middleware to log what is happening:

	transmitter.after(function(packet, next){
		// log the packet to the console
		console.log('packet (' + packet.id + ')');
		console.log([packet.modulator,': ',packet.data,' -> ',packet.frequency].join(''));
		
		next();
	});

### Modulators
A modulator is responsible for sending some data to a frequency.

The SSH modulator will use it's data as an SSH command and pipe the command down the wire.

A modulator is a function that accepts a **packet** argument which is an object with these properties:

It has two core functions:

* complete:		callback for when we have finished (we pass any error normal node style as first argument)

* modulate:		used if we want to pass the packet to another modulator (great for creating modulation 'stacks')

The modulator **MUST** call 'complete' at some point, if it does not then the entire broadcast might get
stuck and bad things will happen (hey, calling one function ain't too much to ask for such a flexible setup : )

Here is an example of a custom modulator that just logs to a file:

	var myCustomModulator = function(packet, complete) {

		var string = packet.data + ' was sent to ' + data.frequency;

		// quasi code coz I'm lazy
		something.appendToFile('/tmp/somefile.txt', string, packet.complete);
		
	};

We can plug-and-play with modulators such that the same recording ends up being interpreted in many different ways
depending on how the transmitter (and it's modulators) are setup.

A good example is for testing, where I replace the standard SSH modulator with a dummy one therefore not needing
to muck around with real SSH connections (the very problem that led me to think about modulators).








### Broadcasting
The eventual aim is to broadcast your recording on a frequency.  The modulator will then use the frequency to deliver
the recording.

In our SSH example we have:

* a transmitter with an SSH modulator
* a recording that will reboot our application
* the frequencies

and a frequency
that represents our 12 application servers (as an array of hostnames).










## Modulators

	var connectionsByHostname = {};

	var getSSHConnection(hostname){
		return connectionsByHostname[hostname] ? 
			connectionsByHostname[hostname] :
			connectionsByHostname[hostname] = new SSHLibraryOfSomeKind(hostname);
	}

	var sshModulator = function(command, hostname, next){

		var sshConnection = getSSHConnection(hostname);

		sshLibrary.sendCommand(command, next);

	}













You use telegraft by broadcasting recordings on particular frequencies.  The recording is a sequence of instructions that when
broadcast is passed through a middleware chain for actioning.

Each middleware has the recording and the frequency to do with what it pleases.  This gives you a very powerful orchestration
framework.

A simple example is to broadcast some SSH commands onto some servers:

	var telegraft = require('telegraft');

	telegraft()
	.record(function(broadcast){
		broadcast.record(telegraft.announce('Testing file creation'));
		broadcast.record('echo "Hello World" > /tmp/telegraft.txt');
		broadcast.wait();
		broadcast.record('rm -f /tmp/telegraft.txt');
	})
	.use(function(handler, data, frequency, next)){

	}
	.transmit('1.2.3.4');

### Recordings
A **recording** is a re-usable function that defines a certain action
to be carried out on a server.  Recordings can use other recordings and because they are just
functions, the composition of them (like loading libraries and config) is up to you.

Each recording function accepts a **broadcast** object and it will use 2 main methods on it:

* **record:** 	logs a command to be carried out on a server
* **wait:** 		waits until the previous commands have finished on ALL servers

Here is an example of a very basic recording that outputs 'Hello World' to a temp file, waits
and then deletes it (it is the same as in the first example but wrapped up in a library function).

	var recordingTest = function(broadcast) {
		broadcast.announce('Testing file creation')
		broadcast.record('echo "Hello World" > /tmp/telegraft.txt');
		broadcast.wait();
		broadcast.record('rm -f /tmp/telegraft.txt');
	};

To broadcast this recording to our localhost (it will just use the local shell):

	telegraft(recordingTest).transmit('localhost')

To broadcast this recording to remote hosts (you must have SSH keys in place):

	telegraft(recordingTest).transmit('remotehost')

### Asynchronous Transmission
The above example is all nice and dandy but a bit pointless on one server - why are we calling wait() in the example above?

The power of telegraft is apparent when you want to broadcast longwave around the world to your whole infrastructure at once.

Imagine we had 50 servers in 5 data centers and wanted to re-build our application on the webservers, modify the
database servers and finally reboot the loadbalancer and webservers in one chunk.

Telegraft co-ordinates this so you are sure that things happen sequentially:

	// our rebuild recording
	var rebuildApplication = function(broadcast) {...}

	// our database update recording
	var modifyDatabase = function(broadcast) {...}

	// our application reboot recording
	var rebootApplication = function(broadcast) {...}

	// our current infrastructure (you can load this from anywhere)
	var servers = {
		'application':['34.65.45.23','34.56.23.45',...],
		'database':['98.65.45.23','98.56.23.45',...],
		'loadbalancer':['214.65.45.23','214.56.23.45',...]
	}

	telegraft(function(broadcast){

		// target the application servers
		broadcast.frequency(servers.application);

		// tell the application servers to rebuild
		broadcast.record(rebuildApplication);

		// wait the all application servers to rebuild
		broadcast.wait();

		// target the database servers
		broadcast.frequency(servers.database);

		// tell the database servers to update 
		// (tunes into their frequency by passing the 2nd arg)
		broadcast.record(modifyDatabase);

		// wait the all database servers are updated
		broadcast.wait();

		// target the whole stack
		broadcast.frequency([
			servers.applications,
			servers.loadbalancer
		]);

		// tell the whole stack to reboot
		broadcast.record(rebootApplication);

	}).use(function(event){

		// the broadcast has a middleware pattern with each actual command
		// being passed down the middleware chain

		// this is a useful place to log what actually happened

	}).transmit();

In the example above, Telegraft has orchestrated an application stack that could be distributed globally.

All failures are removed from the sequence and reported via the logging middleware function which lets you 
hook into any event (via the transmission) and do any reporting you want.

### Transmission Frequency
When you transmit a broadcast - it is on a particular frequency (which is just an array of IP's/hostnames).

The frequency of a broadcast dictates what servers will hear it and you can change the frequency at any time.

For simple broadcasts - it is more common to specify the frequency at the start using the
transmit function:

	// target the same servers for the whole broadcast
	telegraft(function(broadcast){
		broadcast.record('uptime');
	}).transmit(['1.2.3.4', '5.6.7.8']);

You can start off with no frequency and call broadcast.frequency at any time:

	// change the frequency during a recording
	telegraft(function(broadcast){
		broadcast.frequency('1.2.3.4');
		broadcast.record('uptime');
		broadcast.frequency('5.6.7.8');
		broadcast.record('uptime');
	}).transmit();

As soon as you change the frequency any subsequent recordings will be transmitted to it.

### Composition
Recordings are just functions.  The idea is that you can organize recordings and include libraries
of them in any manner you see fit.

When you call broadcast.record - it will behave differently depending on the first argument:

#### String
This is the most basic and low level type of recording, it basically represents a single SSH or SCP
command.

	// send an SSH command down the wire
	broadcast.record('ssh', 'uptime');

	// send an SCP command down the wire
	broadcast.record('scp', '...');

	// assume an SSH command if none given
	broadcast.record('uptime');	

#### Function
This function will be run and given a broadcast object just like any other recording.  This allows
you to group commands and include libraies.

	broadcast.record(function(){
		broadcast.announce('Testing file creation')
		broadcast.record('echo "Hello World" > /tmp/telegraft.txt');
		broadcast.wait();
		broadcast.record('rm -f /tmp/telegraft.txt');
		broadcast.wait();
		broadcast.record(telegraft.archive.service({
			name:'httpd',
			action:'restart'
		})
	})

#### Object
Each item is recorded with the object key as it's announcement - ordering is un-important.

	broadcast.record({}

		'Cleaning Up': function() {
			broadcast.record('rm -rf /tmp/mydata');
		},

		'Restarting Servers': function() {
			broadcast.record(restartServers(['httpd', 'myapp']));
		}

	]);

#### Array
Each item is recorded in order.

	broadcast.announce('Restarting Servers')

	broadcast.record([

		telegraft.archive.service({
			name:'httpd',
			action:'restart'
		}),

		telegraft.archive.service({
			name:'myapp',
			action:'restart'
		})

	]);

### Closure Recordings
It's always a good idea to have your recording functions wrapped in a closure - this lets you
use the same recording but with different data (this is how recordings from the archive work).

Take our example above:

	var recordingTest = function(broadcast) {
		broadcast.record('echo "Hello World" > /tmp/telegraft.txt');
		broadcast.wait();
		broadcast.record('rm -f /tmp/telegraft.txt');
	};

To make this more useful:

	var testFileCreation = function(path, content){
		return function(broadcast){
			broadcast.record('echo "' + content + '" > ' + path);
			broadcast.wait();
			broadcast.record('rm -f ' + path);
		}
	}

	broadcast.record(testFileCreation('/tmp/telegraft.txt', 'Hello World'));

A classic use case might be to make a recording that uploads some new virtual host config file and
then restarts apache.

So we make a closure that accepts the variables and returns the recording function.

	// a closure function that accepts a list of website configs and returns
	// a function that will record everything on the broadcast
	var addVirtualHosts = function(websites) {

		// here is the return function that will be run by telegraft with a broadcast
		return function(broadcast) {

			// we want to add the config files for each website
			for(var i in websites) {
				var website = websites[i];

				var httpConfigDev = '/etc/httpd/sites-available/' + website.name;
				var httpConfigLive = '/etc/httpd/sites-enabled/' + website.name;

				// use an archive recording to upload the virtual host config file for this website
				broadcast.record(telegraft.archive.template({
					source:'./templates/apacheVHost', 
					dest:httpConfigDev,
					vars:{
						host:website.domain,
						document_root:'/home/' + website.name + '/website/www'
					}
				}));

				// symlink the virtual host file so it loads up
				broadcast.record('ln -s ' + httpConfigDev + ' ' + httpConfigLive);
			}

			// we want to wait now for all of the configs to be prepared
			broadcast.wait();

			// now we restart all the servers
			broadcast.record(telegraft.archive.service({
				name:'httpd',
				action:'restart'
			});

		};

	}

To use this:

	broadcast.record(addVirtualHosts([
		{
			name:"bob",
			domain:"bob.com"
		},
		{
			name:"dave",
			domain:"dave.com"
		}

	]));


## Background
Telegraft is a way of remotely configuring servers using SSH.

It is inspired heavily by [Ansible](http://ansible.github.com/)

### Components
Components at play in a telegraft network:

* **Transmitter**: the control server(s) that are broadcasting commands

* **Receiver**: the servers that are recieving commands

* **Recording**: some work to be done to a server

* **Broadcast**: sending out a recording to a particular server

### Usage Flow

1. The Transmitter (**Tx**) is setup

2. SSH access from the **Tx** to the Receivers (**Rx's**) is enabled

3. Recordings are coded and then transmitted to certain Receivers

4. This results in SSH & SCP commands being co-ordinated in sequence

5. The broadcast can then summarize what happened


## Credits
[JQuarry](http://jquarry.com) follow jquarry on twitter [@quarryjs](http://twitter.com/quarryjs)

## Licence
Licenced under the [MIT License](http://opensource.org/licenses/mit)