var telegraft = require('../'),
    async = require('async'),
    _ = require('underscore');



describe('broadcast', function(){
	describe('.encode', function(){

  	it('must create a transmission sequence from the recording', function () {

  			var broadcast = telegraft();

        var sequence = broadcast.record(function() {

          broadcast.record('uptime');
          broadcast.record('cp test test2');
          broadcast.record('cp test test2');

        }).encode('localhost');

        sequence.should.be.a('array');
        sequence.length.should.equal(1);



  	});

  	it('must be able to group a recording into wait block transmission sequences', function () {

  			var broadcast = telegraft();

        var sequence = broadcast.record(function() {

          broadcast.record('uptime');
          broadcast.record('cp test test2');
          broadcast.record('cp test test2');

          broadcast.wait();

          broadcast.record('uptime');
          broadcast.record('cp test test2');
          broadcast.record('cp test test2');


        }).encode('localhost');

        sequence.should.be.a('array');
        sequence.length.should.equal(2);

  	});

  	it('must be able to group a recording into frequency transmission sequences', function () {

  			var broadcast = telegraft();

        var sequence = broadcast.record(function() {

          broadcast.record('uptime');
          broadcast.record('cp test test2');
          broadcast.record('cp test test2');

          broadcast.frequency('1.2.3.4');

          broadcast.record('uptime');
          broadcast.record('cp test test2');
          broadcast.record('cp test test2');


        }).encode('localhost');

        sequence.should.be.a('array');
        sequence.length.should.equal(1);

        var firstSequence = sequence[0];

        firstSequence.should.be.a('array');
        firstSequence.length.should.equal(2);

  	});

  	it('must be able to group a recording into wait block & frequency transmissions', function () {

  			var broadcast = telegraft();

        var sequence = broadcast.record(function() {

          broadcast.record('uptime');
          broadcast.record('cp test test2');
          broadcast.record('cp test test2');

          broadcast.wait();

          broadcast.record('uptime');
          broadcast.record('cp test test2');
          broadcast.record('cp test test2');

          broadcast.frequency('1.2.3.4');

          broadcast.record('uptime');
          broadcast.record('cp test test2');
          broadcast.record('cp test test2');


        }).encode('localhost');

        sequence.should.be.a('array');
        sequence.length.should.equal(2);

        var firstSequence = sequence[0];
        var secondSequence = sequence[1];

        firstSequence.should.be.a('array');
        firstSequence.length.should.equal(1);

        secondSequence.should.be.a('array');
        secondSequence.length.should.equal(2);

  	});

    

  })

})