var telegraft = require('../'),
    _ = require('underscore');



describe('broadcast', function(){
	describe('.record', function(){

		it('must not save undefined actions to the queue', function () {

  			var broadcast = telegraft();

  			broadcast.record('');
  			broadcast.record(null);
  			broadcast.record([]);
  			
  			broadcast.queue().length.should.equal(0);
  			
  	});

  	it('must save string recordings to the queue', function () {

  			var broadcast = telegraft();

  			broadcast.record('test1');
  			broadcast.record('test2');

  			broadcast.queue().length.should.equal(2);

  	});


  	it('must save array recordings to the queue', function () {


  		var broadcast1 = telegraft([
        "uptime",
        function(broadcast) {
          broadcast.record('test2');
          broadcast.record('test3');
        },
        "test3",
        "test4"
      ]);

  		broadcast1.queue().length.should.equal(5);

  	});

  	it('must save object recordings to the queue', function () {
  		var testObject = {
  			"test 1":"uptime",
  			"Test 2": function(broadcast) {
  				broadcast.record('test2');
  				broadcast.record('test3');
  			}
  		};

  		var broadcast = telegraft(testObject);

  		broadcast.queue().length.should.equal(5);

  	});

  	it('must save function recordings to the queue', function () {

  			var testFunction = function(broadcast) {

  				var root = {
  					name:'root',
  					children:[{
  						name:'Layer1 A',
  						children:[{
  							name:'Layer2 A',
  						},{
  							name:'Layer2 B',
  						}]
  					}]
  				};

  				var recurseFunction = function(node) {
  					broadcast.record(node.name);

  					var children = node.children ? node.children : [];
  					
  					_.forEach(children, function(child) {
  						recurseFunction(child);
  					});
  				};

  				recurseFunction(root);
  			};

  			var broadcast = telegraft(testFunction);

  			broadcast.queue().length.should.equal(4);
  			
  	});
  })

})